using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using JALInterfaces;
using System.Reflection;
using System.IO;

namespace TestDrawFactory
{	
	/// <summary>
	/// Summary description for Form1.
	/// Demonstrates the use of growable class factories
	/// </summary>

	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		// class properties

		// default DrawableShape properties
		private Color color= Color.Black;
		private int brushWidth= 2;
		private int size= 50;
		private Point position= new Point(0,0);
		private String currentShape= "Circle";

		// default menu constants
		private readonly int BRUSH_WIDTH_THIN= 1;
		private readonly int BRUSH_WIDTH_MEDIUM= 2;
		private readonly int BRUSH_WIDTH_THICK= 3;
		private readonly int SIZE_SMALL= 20;
		private readonly int SIZE_MEDIUM = 40;
		private readonly int SIZE_LARGE= 60;

		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItemFile;
		private System.Windows.Forms.MenuItem menuItemClear;
		private System.Windows.Forms.MenuItem menuItemExit;
		private System.Windows.Forms.MenuItem menuItemShape;
		private System.Windows.Forms.MenuItem menuItemColor;
		private System.Windows.Forms.MenuItem menuItemColorBlack;
		private System.Windows.Forms.MenuItem menuItemColorRed;
		private System.Windows.Forms.MenuItem menuItemColorGreen;
		private System.Windows.Forms.MenuItem menuItemColorBlue;
		private System.Windows.Forms.MenuItem menuItemSize;
		private System.Windows.Forms.MenuItem menuItemSizeSmall;
		private System.Windows.Forms.MenuItem menuItemSizeMedium;
		private System.Windows.Forms.MenuItem menuItemSizeLarge;
		private System.Windows.Forms.MenuItem menuItemBrush;
		private System.Windows.Forms.MenuItem menuItemBrushThin;
		private System.Windows.Forms.MenuItem menuItemBrushMedium;
		private System.Windows.Forms.MenuItem menuItemBrushThick;
		private System.Windows.Forms.MenuItem menuItemHelp;
		private System.Windows.Forms.MenuItem menuItemHelpAbout;		
		private System.Windows.Forms.MenuItem menuItemPrint;
		private System.Windows.Forms.PrintDialog printDialog1;
		private System.Drawing.Printing.PrintDocument printDocument1;
		private System.Windows.Forms.MenuItem menuItemPrintPreview;
		private System.Windows.Forms.PrintPreviewDialog printPreviewDialog1;		
		private System.Windows.Forms.PageSetupDialog pageSetupDialog1;
		private System.Windows.Forms.MenuItem menuItemPageSetup;

		// create our empty factories
		private TypeBasedDrawableFactory typeFactory= 
			new TypeBasedDrawableFactory();



		// store array of drawings to refresh screen on Invalidate
		// ASSERT only contains IDrawableShapes
		private ArrayList drawHistory= new ArrayList();
		
		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			// add Custom event handlers
			this.MouseUp +=new MouseEventHandler(Form1_MouseUp);

			// add Statically Compiled classes to our factories
			// type based factory
			typeFactory.Add(typeof(Circle));
			typeFactory.Add(typeof(Square));

			// now dynamically load classes in a "plugin" folder which
			// implement JALInterfaces.IDrawableShape 
			Assembly assembly= null;
			try 
			{      
				// tell app where to look for plugins
				AppDomain.CurrentDomain.AppendPrivatePath("plugins");

				//test
				string d="DrawablePlugIns.Triangle,DrawablePlugIns";
				Type myType= Type.GetType(d);
				try 
				{
					object o=System.Activator.CreateInstance(myType);
					//object o= System.Activator.CreateInstance("DrawablePlugIns","DrawablePlugIns.Triangle");
					//System.Console.WriteLine(o);
				}
				catch(Exception e){;}
				// end test

				// get absolute path to our private assemblies
				string path= AppDomain.CurrentDomain.BaseDirectory+"plugins";
				
				// create plugins folder if one does not exist
				DirectoryInfo info= new DirectoryInfo(path);
				if (!info.Exists){ info.Create();}
				// discover all dlls in plugins folder
				string[] dir= Directory.GetFiles(path,"*.dll");
				// iterate over files in folder plugins
				foreach (string s in dir)
				{
					string dll= Path.GetFileNameWithoutExtension(s);
					assembly= Assembly.Load(dll);  // in folder plugins
					Type[] types= assembly.GetTypes();
					foreach(Type t in types) 
					{
						//Console.WriteLine("Type: {0}",t);
						try 			
						{
							// only load into type based factory
							// if type implements IDrawableShape
							if (typeof(IDrawableShape).IsAssignableFrom(t)) 
							{
								typeFactory.Add(t);
							}
						}
						catch(Exception e) 
						{
							Console.WriteLine(e);
						}
					}
				}
				// Done loading factory

				// Populate Shape Menu by
				// iterating over objects in our type based factory
				
				ICollection collection= typeFactory.Keys();
				IEnumerator enumerator= collection.GetEnumerator();
				while (enumerator.MoveNext())
				{
					String menuName= (string)enumerator.Current;
					menuItemShape.MenuItems.Add(menuName,
						new EventHandler(this.shape_Click));
				}

			}
			catch(Exception e) 
			{
				Console.WriteLine(e);
			}
		}

		// Use array of DrawableShape objects to repaint
		// and restore screen state on invalidate. Each object
		// stores its own state and faithfully redraws itself! I believe
		// the framework is smart enough to limit repainting
		// to "dirty" area of screen rather than repaint
		// the entire screen.
		protected override void OnPaint(PaintEventArgs e) 
		{
			Graphics g= e.Graphics;
			foreach (IDrawableShape ds in drawHistory) 
			{
				ds.DrawYourself(g);
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Form1));
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItemFile = new System.Windows.Forms.MenuItem();
			this.menuItemClear = new System.Windows.Forms.MenuItem();
			this.menuItemPrint = new System.Windows.Forms.MenuItem();
			this.menuItemPrintPreview = new System.Windows.Forms.MenuItem();
			this.menuItemExit = new System.Windows.Forms.MenuItem();
			this.menuItemShape = new System.Windows.Forms.MenuItem();
			this.menuItemColor = new System.Windows.Forms.MenuItem();
			this.menuItemColorBlack = new System.Windows.Forms.MenuItem();
			this.menuItemColorRed = new System.Windows.Forms.MenuItem();
			this.menuItemColorGreen = new System.Windows.Forms.MenuItem();
			this.menuItemColorBlue = new System.Windows.Forms.MenuItem();
			this.menuItemSize = new System.Windows.Forms.MenuItem();
			this.menuItemSizeSmall = new System.Windows.Forms.MenuItem();
			this.menuItemSizeMedium = new System.Windows.Forms.MenuItem();
			this.menuItemSizeLarge = new System.Windows.Forms.MenuItem();
			this.menuItemBrush = new System.Windows.Forms.MenuItem();
			this.menuItemBrushThin = new System.Windows.Forms.MenuItem();
			this.menuItemBrushMedium = new System.Windows.Forms.MenuItem();
			this.menuItemBrushThick = new System.Windows.Forms.MenuItem();
			this.menuItemHelp = new System.Windows.Forms.MenuItem();
			this.menuItemHelpAbout = new System.Windows.Forms.MenuItem();
			this.printDialog1 = new System.Windows.Forms.PrintDialog();
			this.printDocument1 = new System.Drawing.Printing.PrintDocument();
			this.printPreviewDialog1 = new System.Windows.Forms.PrintPreviewDialog();
			this.pageSetupDialog1 = new System.Windows.Forms.PageSetupDialog();
			this.menuItemPageSetup = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItemFile,
																					  this.menuItemShape,
																					  this.menuItemColor,
																					  this.menuItemSize,
																					  this.menuItemBrush,
																					  this.menuItemHelp});
			// 
			// menuItemFile
			// 
			this.menuItemFile.Index = 0;
			this.menuItemFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemClear,
																						 this.menuItemPageSetup,
																						 this.menuItemPrint,
																						 this.menuItemPrintPreview,
																						 this.menuItemExit});
			this.menuItemFile.Text = "File";
			// 
			// menuItemClear
			// 
			this.menuItemClear.Index = 0;
			this.menuItemClear.Text = "Clear Screen";
			this.menuItemClear.Click += new System.EventHandler(this.menuItemClear_Click);
			// 
			// menuItemPrint
			// 
			this.menuItemPrint.Index = 2;
			this.menuItemPrint.Shortcut = System.Windows.Forms.Shortcut.CtrlP;
			this.menuItemPrint.Text = "Print Screen";
			this.menuItemPrint.Click += new System.EventHandler(this.menuItemPrint_Click);
			// 
			// menuItemPrintPreview
			// 
			this.menuItemPrintPreview.Index = 3;
			this.menuItemPrintPreview.Text = "Print Preview...";
			this.menuItemPrintPreview.Click += new System.EventHandler(this.menuItemPrintPreview_Click);
			// 
			// menuItemExit
			// 
			this.menuItemExit.Index = 4;
			this.menuItemExit.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
			this.menuItemExit.Text = "E&xit";
			this.menuItemExit.Click += new System.EventHandler(this.menuItemExit_Click);
			// 
			// menuItemShape
			// 
			this.menuItemShape.Index = 1;
			this.menuItemShape.Text = "Shape";
			// 
			// menuItemColor
			// 
			this.menuItemColor.Index = 2;
			this.menuItemColor.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.menuItemColorBlack,
																						  this.menuItemColorRed,
																						  this.menuItemColorGreen,
																						  this.menuItemColorBlue});
			this.menuItemColor.Text = "Color";
			// 
			// menuItemColorBlack
			// 
			this.menuItemColorBlack.Index = 0;
			this.menuItemColorBlack.Text = "Black";
			this.menuItemColorBlack.Click += new System.EventHandler(this.menuItemColorBlack_Click);
			// 
			// menuItemColorRed
			// 
			this.menuItemColorRed.Index = 1;
			this.menuItemColorRed.Text = "Red";
			this.menuItemColorRed.Click += new System.EventHandler(this.menuItemColorRed_Click);
			// 
			// menuItemColorGreen
			// 
			this.menuItemColorGreen.Index = 2;
			this.menuItemColorGreen.Text = "Green";
			this.menuItemColorGreen.Click += new System.EventHandler(this.menuItemColorGreen_Click);
			// 
			// menuItemColorBlue
			// 
			this.menuItemColorBlue.Index = 3;
			this.menuItemColorBlue.Text = "Blue";
			this.menuItemColorBlue.Click += new System.EventHandler(this.menuItemColorBlue_Click);
			// 
			// menuItemSize
			// 
			this.menuItemSize.Index = 3;
			this.menuItemSize.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemSizeSmall,
																						 this.menuItemSizeMedium,
																						 this.menuItemSizeLarge});
			this.menuItemSize.Text = "Size";
			// 
			// menuItemSizeSmall
			// 
			this.menuItemSizeSmall.Index = 0;
			this.menuItemSizeSmall.Text = "Small";
			this.menuItemSizeSmall.Click += new System.EventHandler(this.menuItemSizeSmall_Click);
			// 
			// menuItemSizeMedium
			// 
			this.menuItemSizeMedium.Index = 1;
			this.menuItemSizeMedium.Text = "Medium";
			this.menuItemSizeMedium.Click += new System.EventHandler(this.menuItemSizeMedium_Click);
			// 
			// menuItemSizeLarge
			// 
			this.menuItemSizeLarge.Index = 2;
			this.menuItemSizeLarge.Text = "Large";
			this.menuItemSizeLarge.Click += new System.EventHandler(this.menuItemSizeLarge_Click);
			// 
			// menuItemBrush
			// 
			this.menuItemBrush.Index = 4;
			this.menuItemBrush.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						  this.menuItemBrushThin,
																						  this.menuItemBrushMedium,
																						  this.menuItemBrushThick});
			this.menuItemBrush.Text = "Brush";
			// 
			// menuItemBrushThin
			// 
			this.menuItemBrushThin.Index = 0;
			this.menuItemBrushThin.Text = "Thin";
			this.menuItemBrushThin.Click += new System.EventHandler(this.menuItemBrushThin_Click);
			// 
			// menuItemBrushMedium
			// 
			this.menuItemBrushMedium.Index = 1;
			this.menuItemBrushMedium.Text = "Medium";
			this.menuItemBrushMedium.Click += new System.EventHandler(this.menuItemBrushMedium_Click);
			// 
			// menuItemBrushThick
			// 
			this.menuItemBrushThick.Index = 2;
			this.menuItemBrushThick.Text = "Thick";
			this.menuItemBrushThick.Click += new System.EventHandler(this.menuItemBrushThick_Click);
			// 
			// menuItemHelp
			// 
			this.menuItemHelp.Index = 5;
			this.menuItemHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.menuItemHelpAbout});
			this.menuItemHelp.Text = "Help";
			// 
			// menuItemHelpAbout
			// 
			this.menuItemHelpAbout.Index = 0;
			this.menuItemHelpAbout.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.menuItemHelpAbout.Text = "&About";
			this.menuItemHelpAbout.Click += new System.EventHandler(this.menuItemAbout_Click);
			// 
			// printDocument1
			// 
			this.printDocument1.PrintPage += new System.Drawing.Printing.PrintPageEventHandler(this.printDocument1_PrintPage);
			// 
			// printPreviewDialog1
			// 
			this.printPreviewDialog1.AutoScrollMargin = new System.Drawing.Size(0, 0);
			this.printPreviewDialog1.AutoScrollMinSize = new System.Drawing.Size(0, 0);
			this.printPreviewDialog1.ClientSize = new System.Drawing.Size(400, 300);
			this.printPreviewDialog1.Enabled = true;
			this.printPreviewDialog1.Icon = ((System.Drawing.Icon)(resources.GetObject("printPreviewDialog1.Icon")));
			this.printPreviewDialog1.Location = new System.Drawing.Point(22, 29);
			this.printPreviewDialog1.MinimumSize = new System.Drawing.Size(375, 250);
			this.printPreviewDialog1.Name = "printPreviewDialog1";
			this.printPreviewDialog1.TransparencyKey = System.Drawing.Color.Empty;
			this.printPreviewDialog1.Visible = false;
			// 
			// menuItemPageSetup
			// 
			this.menuItemPageSetup.Index = 1;
			this.menuItemPageSetup.Text = "Page Setup...";
			this.menuItemPageSetup.Click += new System.EventHandler(this.menuItemPageSetup_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 186);
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = "Draw";
			this.Load += new System.EventHandler(this.Form1_Load);

		}
		#endregion
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		
		private void Form1_Load(object sender, System.EventArgs e)
		{
		
		}

		private void menuItemExit_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void menuItemColorBlack_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Black;
			// Menu display may have overwritten drawings 
			// framework calls Invalidate(rect) to repaint screen
		}

		private void menuItemColorRed_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Red;
		}

		private void menuItemColorGreen_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Green;
		}

		private void menuItemColorBlue_Click(object sender, System.EventArgs e)
		{
			this.color= Color.Blue;
		}

		private void menuItemSizeSmall_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_SMALL;
		}

		private void menuItemSizeMedium_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_MEDIUM;
		}

		private void menuItemSizeLarge_Click(object sender, System.EventArgs e)
		{
			this.size= SIZE_LARGE;
		}

		private void menuItemBrushThin_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_THIN;
		}

		private void menuItemBrushMedium_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_MEDIUM;
		}

		private void menuItemBrushThick_Click(object sender, System.EventArgs e)
		{
			this.brushWidth= BRUSH_WIDTH_THICK;
		}

		private void menuItemClear_Click(object sender, System.EventArgs e)
		{
			drawHistory.Clear();
			this.Invalidate(); // clear the entire screen!
		}

		private void menuItemAbout_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show("A Twisted Object Oriented Program by JAL.","About Draw");
		}

		private void shape_Click(object sender, System.EventArgs e)
		{
			currentShape= ((MenuItem)sender).Text;
		}

		// trap mouse up and draw shape at mouse position
		// add DrawableShape to ArrayList drawHistory
		// use this ArrayList to refresh screen on Invalidate
		private void Form1_MouseUp(object sender, MouseEventArgs e)
		{
			Graphics g= Graphics.FromHwnd(this.Handle);
			
			if (e.X < 0 || e.Y < 0) 
			{
				g.DrawString("Invalid Mouse Coordinates.",
					new Font("Arial",10),
					new SolidBrush(Color.Red),
					10,
					10,
					StringFormat.GenericDefault);
				return;
			}

			IDrawableShape ds= typeFactory.GetInstance(currentShape);
			ds.Position= new Point(e.X,e.Y);
			ds.Size= this.size;
			ds.Color= this.color;
			ds.BrushWidth= this.brushWidth;
			ds.DrawYourself(g);
			drawHistory.Add(ds);
		}

		// Add support for printing
		private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			Graphics g= e.Graphics;
			foreach (IDrawableShape ds in drawHistory) 
			{
				ds.DrawYourself(g);
			}
		}

		private void menuItemPrint_Click(object sender, System.EventArgs e)
		{
			printDialog1.AllowSomePages = true;
			printDialog1.ShowHelp = true;
			printDialog1.Document = printDocument1;
			DialogResult result = printDialog1.ShowDialog();
			if (result==DialogResult.OK)
			{
				printDocument1.Print();
			}
		}
		private void menuItemPrintPreview_Click(object sender, System.EventArgs e)
		{
			printPreviewDialog1.Document = printDocument1;
			printPreviewDialog1.ShowDialog();
		}

		private void menuItemPageSetup_Click(object sender, System.EventArgs e)
		{
			pageSetupDialog1.Document = printDocument1;
			pageSetupDialog1.Document.DefaultPageSettings.Color = true;
			pageSetupDialog1.ShowDialog();
		}

}
	
	// create DrawableShapes statically
	class Square : AbstractDrawableShape
	{
		public override void DrawYourself(Graphics g)
		{
			Pen p= new Pen(Color,BrushWidth);
			g.DrawRectangle(p,Position.X, Position.Y,Size,Size);
		}
	}
	class Circle : AbstractDrawableShape
	{
		public override void DrawYourself(Graphics g)
		{
			Pen p= new Pen(Color,BrushWidth);
			g.DrawEllipse(p,Position.X,Position.Y,Size,Size);
		}
	}

	// the class factory
	class TypeBasedDrawableFactory 
	{
		private Hashtable hash= new Hashtable();
		public bool Add(Type t) 
		{
			if ((t != null)&& !hash.Contains(t.Name) && (typeof(IDrawableShape).IsAssignableFrom(t)))
			{
				hash.Add(t.Name,t);
				return true;
			}
			else return false;
		}
		public int Count() 
		{
			return hash.Count;
		}
		public ICollection Keys() 
		{
			return hash.Keys;
		}
		public IDrawableShape GetInstance(string key) 
		{
			if (key == null || !hash.Contains(key)) 
			{
				return null;
			}
			else 
			{
				Type t= (Type)hash[key];
				//if (typeof(IDrawableShape).IsAssignableFrom(t))// safer
				//{
					// dynamically load this class
					return (IDrawableShape)Activator.CreateInstance(t);
				//}
				//else return null;
			}
		}
		public bool Contains(string key) 
		{
			if (key != null) 
			{
				return hash.Contains(key);
			}
			else {return false;}
		}
	}
}
